--from here till a # has been reused from chrispy chars
rawset(_G, "ChrispyChars", {})

local CC = ChrispyChars

local function CheckSlot(item) // this function deliberately errors when a freeslot does not exist
	if _G[item] == nil // this will error by itself for states and objects
		error() // this forces an error for sprites, which do actually return nil for some reason
	end
end

local function SafeFreeslot(...)
	local item = ...
	if item == nil
		return
	end
	
	if pcall(CheckSlot, item)
		print("\131NOTICE:\128 " .. item .. " was not allocated, as it already exists.")
	else
		freeslot(item)
	end
	
	SafeFreeslot(select(2, ...))
end
SafeFreeslot("S_CHRISPY_DUST1", "S_CHRISPY_DUST2", "S_CHRISPY_BUBBLEDUST", "MT_CHRISPY_DUST", "SPR_CHSP")

mobjinfo[MT_CHRISPY_DUST] = {
	spawnstate = S_CHRISPY_DUST1,
	height = mobjinfo[MT_PARTICLE].height,
	radius = mobjinfo[MT_PARTICLE].radius,
	flags = mobjinfo[MT_PARTICLE].flags
}


states[S_CHRISPY_DUST1] = {
	sprite = SPR_NULL,
	tics = 1,
	nextstate = S_CHRISPY_DUST2
}

states[S_CHRISPY_DUST2] = {
	sprite = SPR_CHSP,
	action = A_WaterState,
	var1 = S_CHRISPY_BUBBLEDUST,
	frame = FF_TRANS60
}

states[S_CHRISPY_BUBBLEDUST] = {
	sprite = SPR_BUBL,
	frame = B|FF_TRANS60
}


function CC.SpawnDustRing(mo, speed, thrust, alwaysabove)
	local scale = mo.scale
	local momz = mo.momz
	if alwaysabove
		momz = -P_MobjFlip(mo)*abs($)
	end
	if abs(momz) < scale
		momz = $ < 0 and -scale or scale
	end
		
	local forwardangle = R_PointToAngle2(0, 0, mo.momx, mo.momy)
	local sideangle = forwardangle + ANGLE_90
	local vangle = R_PointToAngle2(0, 0, FixedHypot(mo.momx, mo.momy), momz)
	
	local cosine = cos(vangle)
	local sine = sin(vangle)
	
	local radius = FixedDiv(mo.height, scale) >> 1
	local xspawn = -FixedMul(FixedMul(cos(forwardangle), cosine), radius)
	local yspawn = -FixedMul(FixedMul(sin(forwardangle), cosine), radius)
	local zoffset = -FixedMul(FixedMul(sine, radius), scale)
	
	local hthrust = 0
	local vthrust = 0
	if thrust
		hthrust = FixedMul(thrust, cosine)
		vthrust = FixedMul(thrust, sine)
	end
	
	cosine = FixedMul($, speed)
	sine = FixedMul($, speed)
	
	for i = 1, 16
		local dust = P_SpawnMobjFromMobj(mo, xspawn, yspawn, radius, mo.eflags & (MFE_UNDERWATER | MFE_TOUCHWATER) and MT_MEDIUMBUBBLE or MT_DUST)
		
		local a = i*ANGLE_22h + forwardangle
		
		local forwardthrust = FixedMul(cos(a), sine)
		local sidethrust = FixedMul(sin(a), speed)
		local zthrust = FixedMul(cosine, cos(a))
		
		dust.z = $ + zoffset
		
		P_Thrust(dust, forwardangle, forwardthrust - hthrust)
		P_Thrust(dust, sideangle, sidethrust)
		dust.momz = -zthrust - vthrust
		
		if dust.type == MT_DUST then dust.state = S_CHRISPY_DUST2 end
		dust.fuse = TICRATE/4
		dust.destscale = scale/8
		dust.scalespeed = FixedMul($, scale)
	end
end

addHook("PlayerThink", function(player)
	if (player.mo and player.mo.valid) and player.mo.skin == "phoenix" then
		if player.mo.eflags & MFE_UNDERWATER then
			player.charability = CA_SWIM
		else
			player.charability = CA_NONE
		end
	end
end)


addHook("PlayerThink", function(player)
	if player.mo.skin ~= "phoenix" then
		return
	end
	
	if (player.cmd.buttons & BT_USE) and P_IsObjectOnGround(player.mo) and player.speed == 0 then
		player.height = 30*FRACUNIT
		player.mo.state = S_PLAY_GLIDE_LANDING
		player.pflags = $ | PF_FULLSTASIS
		player.crouchability = true
	else
		player.height = 48*FRACUNIT
		player.chrouchability = false
	end
end)

local function functionname(player)
    if player.mo.skin ~= "phoenix" then
        return
    end
	
	if P_IsObjectOnGround(player.mo) and player.speed > 0 and player.spindelay == 0 then
		player.pflags = $1|PF_SPINNING
		player.mo.state = S_PLAY_ROLL
		S_StartSound(player.mo, sfx_spin)
		player.spindelay = $ + 1*TICRATE
	end
end

addHook("SpinSpecial", functionname)

addHook("AbilitySpecial", function(player)
	local mo = player.mo
    if player.mo.skin == "phoenix" then
        if player.abilityused < 1 and player.dualdelay == 0 and not (player.mo.eflags & MFE_UNDERWATER) and not player.powers[pw_super] then
            player.abilityused = $ + 1
            player.dualdelay = $ + TICRATE/3
            if player.jcurrentability == 0 then -- Let's internally call 0 a Thok.
				P_SetObjectMomZ(player.mo, 12*FRACUNIT)
				player.mo.state = S_PLAY_ROLL
				player.pflags = $|PF_THOKKED|PF_SPINNING
				S_StartSound(player.mo, sfx_s3k47)
				CC.SpawnDustRing(mo, 6*mo.scale, 2*mo.scale)
				player.jcurrentability = 1
			end
		elseif player.powers[pw_super] and player.abilityused < 1 and player.dualdelay == 0 and not (player.mo.eflags & MFE_UNDERWATER)
			player.abilityused = $ + 1
            player.dualdelay = $ + TICRATE/4
			if player.jcurrentability == 0 then -- Let's internally call 0 a Thok.
				P_SetObjectMomZ(player.mo, 15*FRACUNIT)
				player.mo.state = S_PLAY_ROLL
				player.pflags = $|PF_THOKKED|PF_SPINNING
				S_StartSound(player.mo, sfx_s3k47)
				CC.SpawnDustRing(mo, 6*mo.scale, 2*mo.scale)
				player.jcurrentability = 1
			elseif player.jcurrentability == 1 then
				P_SetObjectMomZ(player.mo, 20*FRACUNIT)
				player.mo.state = S_PLAY_ROLL
				player.pflags = $|PF_THOKKED|PF_SPINNING
				S_StartSound(player.mo, sfx_s3k47)
				player.ghosttrail = 1
				S_StartSound(player.mo, sfx_s3k45)
			end
        end
    end
end)		

addHook("PlayerThink", function(player)
	if player.mo.skin == "phoenix" then
		if leveltime % 2 and player.ghosttrail == 1 then
			local ghost = P_SpawnGhostMobj(player.mo)
			ghost.fuse = TICRATE/3
		end
	end
end)

addHook("PlayerThink", function(player)
	if player.mo.skin ~= "phoenix" then
		return
	end
	
	if player.mo.state == S_PLAY_MELEE and (player.mo.eflags & MFE_SPRUNG) or player.mo.state == S_PLAY_MELEE_FINISH and (player.mo.eflags & MFE_SPRUNG) or player.mo.state == S_PLAY_MELEE_LANDING and (player.mo.eflags & MFE_SPRUNG) then
		player.mo.momz = $*5/4
		player.mo.momx = $*5/4
		player.mo.momy = $*5/4
		S_StartSound(player.mo, sfx_sprong)
	end
end)

addHook("PlayerThink", function(player)
    if player.mo.skin == "phoenix" then
        if player.dualdelay == nil then player.dualdelay = 0
        elseif player.dualdelay > 0 then player.dualdelay = $ - 1 end
        if P_IsObjectOnGround(player.mo) then
			player.abilityused = 0
			player.meleeused = 0
			player.jcurrentability = 0
			player.ghosttrail = 0
		end
    end
end)

addHook("PlayerThink", function(player)
    if player.mo.skin ~= "phoenix" then
        return
    end
    
    if (player.cmd.buttons & BT_CUSTOM1) and P_IsObjectOnGround(player.mo) and player.speed < 6 and player.spindelay == 0 and not player.powers[pw_super] then
        player.pflags = $ |PF_THOKKED|PF_SPINNING
        player.mo.state = S_PLAY_MELEE
		player.mo.tics = 2
		player.spindelay = TICRATE/2
		P_Thrust(player.mo, player.mo.angle, 3*FRACUNIT)
		P_SetObjectMomZ(player.mo, 2*FRACUNIT)
		S_StartSound(player.mo, sfx_s3k8b)
	elseif player.speed > 5 and (player.cmd.buttons & BT_CUSTOM1) and P_IsObjectOnGround(player.mo) and player.spindelay == 0 and not player.powers[pw_super] then
        player.mo.state = S_PLAY_MELEE
		P_SetObjectMomZ(player.mo, 3*FRACUNIT)
		player.pflags = $1|PF_SPINNING
		player.mo.tics = 2
		S_StartSound(player.mo, sfx_s3k8b)
		player.spindelay = 1*TICRATE
		P_Thrust(player.mo, player.mo.angle, 6*FRACUNIT)
	elseif player.meleeused < 3 and (player.cmd.buttons & BT_CUSTOM1) and not P_IsObjectOnGround(player.mo) and player.spindelay == 0 and not player.powers[pw_super] then
		player.mo.state = S_PLAY_MELEE
		P_SetObjectMomZ(player.mo, 2*FRACUNIT, true)
		player.pflags = $1|PF_SPINNING|PF_THOKKED
		player.mo.tics = 2
		player.meleeused = $ + 1
		S_StartSound(player.mo, sfx_s3k42)
		player.spindelay = 1*TICRATE
		P_Thrust(player.mo, player.mo.angle, 1*FRACUNIT)
		player.mo.color = player.skincolor
		player.mo.colorized = false
		player.ghosttrail = 0
	end
end)

addHook("PlayerThink", function(player)
    if player.mo.skin == "phoenix" then
        if player.spindelay == nil then player.spindelay = 0
        elseif player.spindelay > 0 then player.spindelay = $ - 1 end
	end
end)

/*addHook("PlayerThink", function(player)
	if player.mo.skin == "phoenix" then
		if leveltime % 4 >= 2 then
			player.mo.color = SKINCOLOR_BLUE -- Replace this with whatever color you want
			player.mo.colorized = true
		else
			player.mo.color = player.skincolor
			player.mo.colorized = false
		end
	end
end)*/

addHook("PlayerThink", function(player)
    if player.mo.skin ~= "phoenix" then
        return
    end
	
	if P_IsObjectOnGround(player.mo) then
		if (player.cmd.buttons & BT_CUSTOM3) then
			player.mo.state = S_PLAY_FACEPLANT
			player.pflags = $ | PF_STASIS
		end
	end
end)

addHook("PlayerThink", function(player)
    if player.mo.skin ~= "phoenix" then
        return
    end
	
	if P_IsObjectOnGround(player.mo) then
		if (player.cmd.buttons & BT_TOSSFLAG) then
			player.mo.state = S_PLAY_HEHASDRIP
		end
	end
end)

addHook("PlayerThink", function(player)
    if player.mo.skin ~= "phoenix" then
        return
    end
	
	if player.mo.state == S_PLAY_HEHASDRIP then
		player.pflags = $ | PF_STASIS
	end
end)

addHook("PlayerThink", function(player)
    if player.mo.skin ~= "phoenix" then
        return
    end
	
	if player.mo.state == S_PLAY_FACEPLANT then
		player.pflags = $ | PF_STASIS
	end
end)

addHook("PlayerThink", function(player)
    if player.mo.skin ~= "phoenix" then
		return
	end
	
	if player.powers[pw_super] then
		player.mo.colorized = true
	else
		player.mo.colorized = false
	end
end)

addHook("PlayerThink", function(player)
    if player.mo.skin ~= "phoenix" then
		return
	end
	
	if player.hyper and player.hyper.capable and player.powers[pw_super] then
		player.normalspeed = 50*FRACUNIT
		player.jumpfactor = 2*FRACUNIT
		player.mo.colorized = true
	end
end)

addHook("PlayerThink", function(player)
    if player.mo.skin ~= "phoenix" then
		return
	end
	
	if player.padashmodet == false then
		if player.dashmode == 3*TICRATE and P_IsObjectOnGround(player.mo) then
			S_StartSound(player.mo, sfx_s3k81)
			P_SpawnThokMobj(player)
			S_StartSound(player.mo, sfx_s3k46)
			P_Thrust(player.mo,player.mo.angle,5*FRACUNIT)
			local thokko = P_SpawnMobjFromMobj(player.mo, 0, 0, player.mo.scale * 20, MT_THOKKO)
			thokko.angle = player.mo.angle + ANGLE_90
			thokko.fuse = 7
			thokko.scale = 2*FRACUNIT/3
			thokko.destscale = 10*FRACUNIT
			thokko.colorized = true
			thokko.color = player.mo.color
			player.thokeffects = true
		else
			player.thokeffects = false
		end
	end
end)

addHook("PlayerThink", function(player)
    if player.mo.skin == "phoenix" then
		if player.padashmodet == false then
			if player.dashmode > 3*TICRATE then
				P_SpawnThokMobj(player)
			end
		end
	end
end)

addHook("PlayerThink", function(player)
    if player.mo.skin == "phoenix" then
		if player.padashmodet == true then
			player.charflags = $ & ~SF_DASHMODE
			player.charflags = $ | SF_RUNONWATER
		else
			player.charflags = $ | SF_DASHMODE
			player.charflags = $ & ~SF_RUNONWATER
		end
	end
end)

addHook("PlayerThink", function(player)
    if player.mo.skin ~= "phoenix"
        return
    end
    
    if player.crouchability == true and (player.cmd.buttons & BT_JUMP) and player.spindelay == 0 and not player.powers[pw_super]
		player.pflags = $|PF_SPINNING
		player.spindelay = $ + TICRATE/2
		S_StartSound(player.mo,sfx_zoom)
		P_SetObjectMomZ(player.mo, 1*FRACUNIT)
		P_InstaThrust(player.mo, player.mo.angle, 34*FRACUNIT)
		player.mo.state = S_PLAY_ROLL
		player.crouchability = false
	end
end)